/* libiir/src/libiir++/200_Filter.cc
 * 
 *  Copyright: ©2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



Filter::Filter(const Coeff& coeff)
{
    coeff_.push_back(new Coeff(coeff));
    fi_ = iir_filter_new(coeff.cc_);
}



Filter::Filter(const Filter& other)
{
    for(int pos = 0, max = other.coeff_.size(); pos < max; ++pos) {
        Coeff* cc = new Coeff(*other.coeff_[pos]);
        coeff_.push_back(cc);

        if(!pos) fi_ = iir_filter_new(cc->cc_);
        else iir_filter_chain(fi_, cc->cc_);
    }
}



Filter::Filter(const char* desc)
{
    fi_ = iir_parse(desc);
    if(!fi_) return;

    for(int pos = 0, max = iir_filter_coeff_sets(fi_); pos < max; ++pos) {
        struct iir_coeff_t* c = iir_filter_coeff_set(fi_, pos);
        coeff_.push_back(new Coeff(c));
        iir_coeff_free(c);
    }
}



Filter::~Filter()
{
    for(int pos = 0, max = coeff_.size(); pos < max; ++pos) delete coeff_[pos];
    iir_filter_free(fi_);
}



bool
Filter::valid() const
{
    return fi_;
}



double
Filter::f(double x)
{
    return iir_filter(fi_, x);
}



int
Filter::numCoeffSet() const
{
    return coeff_.size();
}



const Coeff&
Filter::getCoeffSet(int idx) const
{
    return *(coeff_[idx]);
}



void
Filter::chainCoeffSet(const Coeff& coeff)
{
    coeff_.push_back(new Coeff(coeff));
    iir_filter_chain(fi_, coeff.cc_);
}



std::complex<double>
Filter::response(double freq) const
{
    return iir_response(fi_, freq);
}



std::vector<double>
Filter::responseMinus3dB(double gain) const
{
    double* cf;
    int num_cf, i;

    iir_response_minus_3dB(fi_, gain, &cf, &num_cf);
    std::vector<double> ret(num_cf);
    for(i = 0; i < num_cf; ++i) ret[i] = cf[i];
    free(cf);
    return ret;
}



/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=cpp.doxygen
*/
